package com.rma.epstopik_bengaliword;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.SearchView;
import androidx.core.content.ContextCompat;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.speech.tts.TextToSpeech;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ForegroundColorSpan;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class DictioneryActivity extends AppCompatActivity {

   private ListView dictionaryListView;
 private SearchView searchView;
 TextView TextS,TextSS;
 ImageView imageView;

    public static String korean;

    public static String Bengali;

    private ArrayAdapter<String> adapter1;

    private static final int TOTAL_CHAPTERS = 49;

    private final List<String[]> chapterLists = new ArrayList<>();

   private TextToSpeech tts;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().setStatusBarColor(ContextCompat.getColor(this,R.color.status));
        setContentView(R.layout.activity_dictionery);
        dictionaryListView=findViewById(R.id.dicList);
        searchView=findViewById(R.id.searchView);
        TextS=findViewById(R.id.TextS);
        TextSS=findViewById(R.id.TextSS);
        imageView=findViewById(R.id.imageView);


        for (int i = 1; i <= TOTAL_CHAPTERS; i++) {
            int resId = getResources().getIdentifier("chapter_" + i, "array", getPackageName());
            if (resId != 0) {
                chapterLists.add(getResources().getStringArray(resId));
            }
        }

// Concatenate all chapter arrays
        String[] concatenatedArray = concatenatedArray(chapterLists.toArray(new String[0][]));

        adapter1=new CustomArrayAdapter(this, soup.neumorphism.R.layout.support_simple_spinner_dropdown_item, concatenatedArray);
        dictionaryListView.setAdapter(adapter1);

        tts = new TextToSpeech(getApplicationContext(), status -> {
            if (status == TextToSpeech.SUCCESS) {
                tts.setLanguage(Locale.KOREA);
                tts.setSpeechRate(0.7f);
            } else {
                Toast.makeText(DictioneryActivity.this, "Text-to-Speech initialization failed", Toast.LENGTH_SHORT).show();
            }
        });

        dictionaryListView.setOnItemLongClickListener((parent, view, position, id) -> {
            String word = (String) adapter1.getItem(position);
            String korean = word.split("➝")[0];
            tts.speak(korean, TextToSpeech.QUEUE_FLUSH, null, null);
            return false;
        });

        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
                // Handle query submission if needed
                return false;
            }
            @Override
            public boolean onQueryTextChange(String newText) {
                if (newText.isEmpty()) {
                    TextS.setVisibility(View.VISIBLE);
                    imageView.setVisibility(View.GONE);
                    TextSS.setVisibility(View.GONE);
                    dictionaryListView.setVisibility(View.GONE);
                    adapter1.getFilter().filter(null); // Reset the filter
                } else {
                    boolean isMatchFound = false;
                    for (int i = 0; i < adapter1.getCount(); i++) {
                        String word = adapter1.getItem(i);
                        if (word != null && word.toUpperCase(Locale.getDefault()).contains(newText.toUpperCase(Locale.getDefault())) && word.toLowerCase(Locale.getDefault()).contains(newText.toLowerCase(Locale.getDefault()))) {
                            isMatchFound = true;
                            break;
                        }
                    }
                    if (isMatchFound) {
                        TextS.setVisibility(View.GONE);
                        imageView.setVisibility(View.GONE);
                        TextSS.setVisibility(View.GONE);
                        dictionaryListView.setVisibility(View.VISIBLE);
                        adapter1.getFilter().filter(newText);
                    } else {
                        TextSS.setVisibility(View.VISIBLE);
                        imageView.setVisibility(View.VISIBLE);
                        dictionaryListView.setVisibility(View.GONE);
                    }
                }
                return true;
            }


        });
    }
    private String[] concatenatedArray(String[][] arrays) {
        int totalLength = 0;
        for (String[] array : arrays) {
            totalLength += array.length;
        }
        String[] result = new String[totalLength];
        int offset = 0;
        for (String[] array : arrays) {
            System.arraycopy(array, 0, result, offset, array.length);
            offset += array.length;
        }
        return result;
    }
    private class CustomArrayAdapter extends ArrayAdapter<String> {
        public CustomArrayAdapter(Context context, int resource, String[] objects) {
            super(context, resource, objects);
        }
        @NonNull
        @Override
        public View getView(int position, @Nullable View convertView, @NonNull ViewGroup parent) {
            View view = super.getView(position, convertView, parent);
            TextView textView = (TextView) view.findViewById(android.R.id.text1);

            // Get the current word
            String word = getItem(position);

            // Get the search query
            String query = searchView.getQuery().toString();

            // If there's a search query and the word contains it, colorize the matching part
            if (!query.isEmpty() && word != null && word.toLowerCase(Locale.getDefault()).contains(query.toLowerCase(Locale.getDefault()))) {
                int startPos = word.toLowerCase(Locale.getDefault()).indexOf(query.toLowerCase(Locale.getDefault()));
                int endPos = startPos + query.length();

                Spannable spannable = new SpannableString(word);
                spannable.setSpan(new ForegroundColorSpan(Color.BLUE), startPos, endPos, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
                textView.setText(spannable, TextView.BufferType.SPANNABLE);
            } else {
                textView.setText(word);
            }

            return view;
        }
    }

    @Override
    protected void onDestroy() {
        if (tts != null) {
            tts.stop();
            tts.shutdown();
        }
        super.onDestroy();
    }
}