package com.rma.epstopik_bengaliword.PdfDownloader;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import com.rma.epstopik_bengaliword.DownloadHelper;
import com.rma.epstopik_bengaliword.R;
import com.rma.epstopik_bengaliword.PDFViewActivity;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class Helper {
    public static final int REQUEST_ID_MULTIPLE_PERMISSIONS = 1005;

    public void DialogBox(Activity context, String pdfUrl) {
        if (isValidURL(pdfUrl)) {
            final AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(context);
            View myLayout = context.getLayoutInflater().inflate(R.layout.dialog_box_pdf, null);
            alertDialogBuilder.setView(myLayout);

            final AlertDialog alertDialog = alertDialogBuilder.create();
            alertDialog.setCancelable(true);

            TextView tvOffline = myLayout.findViewById(R.id.tvOffline);
            TextView warningText = myLayout.findViewById(R.id.warning);
            ImageView OfflineImages = myLayout.findViewById(R.id.offlineImage);
            LinearLayout clickDownload = myLayout.findViewById(R.id.ClickDownload);

            final String appPackageName = context.getPackageName();
            final String filePath = "/storage/emulated/0/Android/data/" + appPackageName + "/files/Eps Topik - Bengali Word/" + getFileNameFromUrl(pdfUrl);

            if (doesFileExist(filePath)) {
                tvOffline.setText("View");
                OfflineImages.setImageDrawable(ContextCompat.getDrawable(context, R.drawable.offline));
                warningText.setVisibility(View.GONE);
                clickDownload.setOnClickListener(v -> {
                    showPDF(context, pdfUrl);
                    alertDialog.dismiss();
                });
            } else {
                clickDownload.setOnClickListener(v -> {
                    downloadPDF(context, pdfUrl);
                    alertDialog.dismiss();
                });
            }
            myLayout.findViewById(R.id.btnCancel).setOnClickListener(v -> alertDialog.dismiss());
            alertDialog.show();
        } else {
            // Show AlertDialog for invalid PDF URL
            showInvalidURLDialog(context);
        }
    }

    // Method to show AlertDialog for invalid PDF URL
    private void showInvalidURLDialog(Activity context) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle("Server error");
        builder.setMessage("Please stay connected. We will be fixed this issue very soon.");
        builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
            }
        });
        builder.setCancelable(false);
        builder.show();
    }

    private boolean isValidURL(String url) {
        return url != null && !url.isEmpty() && (url.startsWith("http://") || url.startsWith("https://"));
    }

    private void downloadPDF(Activity context, String pdfUrl) {
        if (isStoragePermissionGranted(context)) {
            if (pdfUrl != null && !pdfUrl.isEmpty()) {
                ConnectivityManager connectivityManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
                NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();

                if (networkInfo != null && networkInfo.isConnected()) {
                    DownloadHelper downloadHelper = new DownloadHelper(context, pdfUrl, getFileNameFromUrl(pdfUrl), new DownloadListener() {
                        @Override
                        public void onDownloadComplete() {
                            Toast.makeText(context, "Successfully Downloaded", Toast.LENGTH_SHORT).show();
                            showPDF(context, pdfUrl);
                        }

                        @Override
                        public void onDownloadFailed() {
                            Toast.makeText(context, "Download failed ! Please try again", Toast.LENGTH_SHORT).show();
                        }
                    });

                    if (downloadHelper.isValidURL()) {
                        downloadHelper.startDownload();
                    } else {
                        Toast.makeText(context, "Invalid PDF URL", Toast.LENGTH_SHORT).show();
                    }

                } else {
                    showNoInternetDialog(context, pdfUrl);
                }
            } else {
                Toast.makeText(context, "Error ! Invalid PDF URL", Toast.LENGTH_SHORT).show();
            }
        } else {
            Toast.makeText(context, "Permission Denied", Toast.LENGTH_SHORT).show();
        }
    }




    private void showPDF(Activity context, String pdfUrl) {
        final String appPackageName = context.getPackageName();
        final String filePath = "/storage/emulated/0/Android/data/" + appPackageName + "/files/Eps Topik - Bengali Word/" + getFileNameFromUrl(pdfUrl);
        Intent myIntent = new Intent(context, PDFViewActivity.class);
        myIntent.putExtra("pdf", filePath);
        context.startActivity(myIntent);
    }

    public static boolean doesFileExist(String filePath) {
        File file = new File(filePath);
        return file.exists();
    }

    public String getFileNameFromUrl(String url) {
        String fileName = "";
        if (url != null && !url.isEmpty()) {
            int lastIndex = url.lastIndexOf('/');
            if (lastIndex != -1 && lastIndex < url.length() - 1) {
                fileName = url.substring(lastIndex + 1);
            }
        }
        return fileName;
    }

    private boolean isStoragePermissionGranted(Activity activity) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            int READ_MEDIA_IMAGES = ContextCompat.checkSelfPermission(activity, Manifest.permission.READ_MEDIA_IMAGES);
            int READ_MEDIA_AUDIO = ContextCompat.checkSelfPermission(activity, Manifest.permission.READ_MEDIA_AUDIO);


            List<String> listPermissionsNeeded = new ArrayList<>();

            if (READ_MEDIA_IMAGES != PackageManager.PERMISSION_GRANTED) {
                listPermissionsNeeded.add(Manifest.permission.READ_MEDIA_IMAGES);
            }
            if (READ_MEDIA_AUDIO != PackageManager.PERMISSION_GRANTED) {
                listPermissionsNeeded.add(Manifest.permission.READ_MEDIA_AUDIO);
            }
            if (!listPermissionsNeeded.isEmpty()) {
                ActivityCompat.requestPermissions(activity, listPermissionsNeeded.toArray(new String[listPermissionsNeeded.size()]), REQUEST_ID_MULTIPLE_PERMISSIONS);
                return false;
            }
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            int WRITE_EXTERNAL_STORAGE = ContextCompat.checkSelfPermission(activity, Manifest.permission.WRITE_EXTERNAL_STORAGE);
            List<String> listPermissionsNeeded = new ArrayList<>();
            if (WRITE_EXTERNAL_STORAGE != PackageManager.PERMISSION_GRANTED) {
                listPermissionsNeeded.add(Manifest.permission.WRITE_EXTERNAL_STORAGE);
            } else if (!listPermissionsNeeded.isEmpty()) {
                ActivityCompat.requestPermissions(activity, listPermissionsNeeded.toArray(new String[listPermissionsNeeded.size()]), REQUEST_ID_MULTIPLE_PERMISSIONS);
                return false;
            }
        }
        return true;
    }

    private void showNoInternetDialog(Activity context, String pdfUrl) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle("No Internet Connection");
        builder.setMessage("You have no internet connection. Please check your internet connection.");
        builder.setPositiveButton("Retry", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                if (isNetworkAvailable(context)) {
                    downloadPDF(context, pdfUrl);
                } else {
                    showNoInternetDialog(context, pdfUrl);
                }
            }
        });
        builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
            }
        });
        builder.setCancelable(false);
        builder.show();
    }

    private boolean isNetworkAvailable(Activity context) {
        ConnectivityManager connectivityManager = (ConnectivityManager) context.getSystemService(context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();
        return networkInfo != null && networkInfo.isConnected();
    }


}
