package com.rma.epstopik_bengaliword.TTS;

import android.content.Context;
import android.speech.tts.TextToSpeech;
import android.widget.Toast;

import java.util.Locale;

public class TTSSingleton {

    private static TTSSingleton instance;
    private TextToSpeech textToSpeech;
    private boolean isInitialized = false;

    private TTSSingleton(Context context) {
        textToSpeech = new TextToSpeech(context, status -> {
            if (status == TextToSpeech.SUCCESS) {
                int result = textToSpeech.setLanguage(Locale.KOREAN);
                if (result == TextToSpeech.LANG_MISSING_DATA || result == TextToSpeech.LANG_NOT_SUPPORTED) {
                    Toast.makeText(context, "Language not supported", Toast.LENGTH_SHORT).show();
                }
                textToSpeech.setSpeechRate(.7f);
                isInitialized = true;
            } else {
                Toast.makeText(context, "Text-to-Speech initialization failed", Toast.LENGTH_SHORT).show();
            }
        });
    }

    public static TTSSingleton getInstance(Context context) {
        if (instance == null) {
            instance = new TTSSingleton(context);
        }
        return instance;
    }

    public void speak(String text,Context context) {
        if (isInitialized) {
            textToSpeech.speak(text, TextToSpeech.QUEUE_FLUSH, null, null);
        } else {
            Toast.makeText(context, "Please try again later", Toast.LENGTH_SHORT).show();
        }
    }

    public void shutdown() {
        if (textToSpeech != null) {
            textToSpeech.stop();
            textToSpeech.shutdown();
            isInitialized = false;
        }
        instance = null; // Reset the instance
    }
}
